class PcScreenCaptureVideo extends PcScreen
{
  constructor(props)
  {
    super(props);
    this.state = {flashVal:props.shoots, displayCounter:true}
    this.recordingBar = React.createRef();
    this.counter = React.createRef();
    this.flash = React.createRef();
    this.steps = React.createRef();
    this.title = React.createRef();
    this.pretitle = React.createRef();

    this.canStop = false;
  }

  onCount(n)
  {
    console.log("count " + n);
  }

  onEndCount()
  {
    this.counter.current.hide();
    this.title.current.show();
    this.pretitle.current.hide();
    this.startRecording();
    PcCameraControl.startVideoRecording();
    setTimeout(()=>{this.canStop = true}, 1000);
  }

  startRecording()
  {
    this.recordingBar.current.start();
  }

  onClickEnd()
  {
    if (!this.canStop) return;

    this.canStop = false;
    PcCameraControl.stopVideoRecording();
    this.recordingBar.current.stop();
    this.title.current.hide();
    this.pretitle.current.show();
    this.props.goNextScreen();
  }

  onTick(n)
  {

  }

  onEndRecording()
  {
    this.onClickEnd();
  }

  componentDidUpdate()
  {
    if (this.props.active)
    {
      this.title.current.hide();

      if (this.counter.current)
      {
        this.counter.current.initCount();
        this.counter.current.show();
      }
    }
  }

  render()
  {
    if (!this.props.active) return null;

    let cameraWidth, cameraHeight, cameraTop, cameraLeft, titleY;
    let poseRatio = 16/9;
    let text = PcConfig.screen.menu.themes[PcConfig.getMemory("currentTheme")].textCapture;

    if (PcMode.mode == "videobooth")
    {
      cameraWidth = 1100 * PcConfig.renderSize;
      cameraHeight = 1100 / poseRatio * PcConfig.renderSize;
      cameraTop = (this.props.height-cameraHeight)/2;
      cameraLeft = (this.props.width-cameraWidth)/2;

      if (text !="") cameraTop -= 50*PcConfig.renderSize;
    }
    else if (PcMode.mode == "videokiosk")
    {
      cameraWidth = 1080 * PcConfig.renderSize;
      cameraHeight = 1080 / poseRatio * PcConfig.renderSize;
      cameraTop = 0;
      cameraLeft = (this.props.width-cameraWidth)/2;
    }

    return e('div',
      {
        className:this.state.className,
        index:this.props.index,
        style:
        {
          display: (this.props.active ? 'block' : 'none'),
          fontFamily:PcConfig.scenario.fontFamily,
          animation:this.props.noAnimation?"":this.state.animationStyle
        },
      },
      e(PcMedia,
        {
          content:this.props.foreground,
          width:640*2*PcConfig.renderSize,
          height:360*2*PcConfig.renderSize,
          x:(1920-(640*2))/2*PcConfig.renderSize,
          y:(1080-(360*2))/2*PcConfig.renderSize,
          position:"absolute",
        }
      ),
      e(PcRecordingBar,
        {
          ref:this.recordingBar,
          onTick:(n)=>this.onTick(n),
          onEndRecording:()=>this.onEndRecording(),
          width:"100%",
          height:"5%",
          backgroundColor:PcConfig.scenario.actionButtonBackgroundColor,
          textColor:PcConfig.scenario.actionButtonTextColor,
          time:PcConfig.screen.capture.recordingTime,
          showDelay:"1s"
        }
      ),
      e(PcLabel,
        {
          ref: this.pretitle,
          label: PcConfig.screen.capture.introTextVideo,
          width:"90%",
          height:"15%",
          x:"5%",
          y:0,
          fontSize:PcConfig.scenario.fontSizeBig,
          color: PcConfig.scenario.fontColor,
          valign: "middle",
          showDelay:"0.1s"
        }
      ),
      e(PcCamera,
        {
          ref:this.camera,
          width:cameraWidth,
          height:cameraHeight,
          x:cameraLeft,
          y:cameraTop,
          foreground:PcConfig.screen.menu.themes[PcConfig.getMemory("currentTheme")].maskPreviewPicture,
          animation:"elementMoveDownAndFadeIn 0s ease-out 0s 1 normal forwards",
          animationDelay:0,
          active:this.props.active,
        }
      ),
      e(PcLabel,
        {
          ref: this.title,
          label: PcConfig.screen.capture.textVideo,
          width:cameraWidth,
          height:cameraHeight,
          x:cameraLeft,
          y:cameraTop + 50*PcConfig.renderSize,
          fontSize:PcConfig.scenario.fontSize,
          color: "#ffffff",
          valign: "middle",
          spacingV : -50*PcConfig.renderSize,
          shadow: 3*PcConfig.renderSize + "px " + 3*PcConfig.renderSize + "px " + 5*PcConfig.renderSize + "px rgba(0, 0, 0, .5)",
          showDelay:"0.1s"

        }
      ),
      e(PcLabel,
        {
          label: text,
          title:true,
          width:"90%",
          x:"5%",
          y:cameraTop+cameraHeight+(25*PcConfig.renderSize),
          fontSize:PcConfig.scenario.fontSize,
          color: PcConfig.scenario.fontColor,
          valign: "top",
          showDelay:"0.1s"
        }
      ),
      e(PcCounter,
        {
          ref:this.counter,
          active:this.props.active,
          display:this.state.displayCounter,
          startVal:4,
          delay:1000,
          onCount:(n)=>this.onCount(n), // FIXME: async() => await? 
          onEndCount:()=>this.onEndCount(),
          width:cameraWidth,
          height:cameraHeight*0.75,
          x: cameraLeft,
          y: cameraTop,
          color:PcConfig.scenario.counterTextColor,
          showDelay:"1s"
        }
      ),
      e('div',
        {
          ref:this.flash,
          className:"flash",
          style:
          {
            position:"absolute",
            display:"none",
            backgroundColor:PcConfig.scenario.startButtonTextColor,
            width:"100%",
            height:"100%",
            color:PcConfig.scenario.startButtonTextColor,
            animation: "flashFadeOut 750ms ease-out 0s 1 forwards",
          }
        }
      ),
      e('div',
        {
          style:
            {
              position:"absolute",
              width:"100%",
              top:"85%",
              height:"15%",
              display:"inline-flex",
              justifyContent: "center",
              alignItems:"center",
              alignContent:"space-around",
            }
        },
        e(PcButton,
          {
            label:PcConfig.screen.capture.endButton.title,
            onClick:()=>this.onClickEnd(),
            borderColor:PcConfig.scenario.actionButtonBorderColor,
            backgroundColor:PcConfig.scenario.actionButtonBackgroundColor,
            fontColor:PcConfig.scenario.actionButtonTextColor,
            showDelay:"5s"
          }
        )
      )
    );
  }
}
