#!/usr/bin/python3
import urllib.request
import argparse
import sys
from time import perf_counter, sleep
import simpleflock
from common import *

def parse_args():
    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument('--help', '-h', help='show this help message and exit', action='store_true')
    parser.add_argument('--period', '-p', help='define request periodicity in seconds (default: None means oneshot)', type=float, default=None)
    parser.add_argument('--url', '-u', help='define url to get', type=str, required=True)
    parser.add_argument('--lock_file', '-k', help='lock on lockfile', default=None)
    parser.add_argument('--lock_timeout', '-t', help='lockfile timeout', default=0)
    parser.add_argument('--verbose', '-v', help='verbose', action='store_true', default=False)
    args = parser.parse_args()

    if args.help:
        parser.print_help(sys.stdout)
        sys.exit(0)
    return args

def dbg(doit, log):
    if (doit):
        print(log)

def get_time():
    return perf_counter()


class URLRequester():
    def __init__(self, args):
        self.args = args

    def loop(self):
        while True:
            req_start_time = get_time()
            contents = urllib.request.urlopen(self.args.url).read()
            dbg(self.args.verbose, "contents len:" + str(len(contents)))
            req_end_time = get_time()
            req_duration = req_end_time - req_start_time
            if self.args.period is None:
                break
            time_to_sleep = self.args.period - req_duration
            if time_to_sleep < 0:
                print(f"request to {self.args.url} took too long! {req_duration} > {time_to_sleep}")
                continue
            dbg(self.args.verbose, f"sleeping {time_to_sleep} seconds")
            sleep(time_to_sleep)


def main():
    args = parse_args()
    req = URLRequester(args)
    if args.lock_file:
        try:
            dbg(args.verbose, f"trying to lock on {args.lock_file} with timeout {args.lock_timeout}")
            with simpleflock.SimpleFlock(args.lock_file, args.lock_timeout):
                req.loop()
        except BlockingIOError as ex:
            dbg(args.verbose, f"cannot lock {args.lock_file}: already in use")
            sys.exit(1)
    else:
        req.loop()


main()
